import { Map } from 'immutable';
import { loop, isLoop, getModel, getCmd, Cmd } from 'redux-loop'; //a helper to merge a bunch of reducers together with no shared root state
//simpler version of mergeChildReducers for when the parent does nothing (like app root)

export default function combineReducers(childMap) {
  return function () {
    var rootState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : Map();
    var action = arguments.length > 1 ? arguments[1] : undefined;

    for (var _len = arguments.length, args = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
      args[_key - 2] = arguments[_key];
    }

    var keys = Object.keys(childMap);
    var cmds = [];
    var newState = rootState.withMutations(function (state) {
      keys.forEach(function (key) {
        var childReducer = childMap[key];

        if (!childReducer) {
          state["delete"](key);
          return;
        }

        var currentChild = childReducer.apply(void 0, [state.get(key), action].concat(args));

        if (isLoop(currentChild)) {
          cmds.push(getCmd(currentChild));
          currentChild = getModel(currentChild);
        }

        state.set(key, currentChild);
      });
    });
    return loop(newState, getListCmdIfNeeded(cmds));
  };
}

function getListCmdIfNeeded(cmds) {
  switch (cmds.length) {
    case 0:
      return Cmd.none;

    case 1:
      return cmds[0];

    default:
      return Cmd.list(cmds);
  }
}